@@ -6,6 +6,8 @@ OpenStack Cloud Module
 OpenStack is an open source project that is in use by a number a cloud
 providers, each of which have their own ways of using it.
 
+:depends: libcloud >- 0.13.2
+
 OpenStack provides a number of ways to authenticate. This module uses password-
 based authentication, using auth v2.0. It is likely to start supporting other
 methods of authentication provided by OpenStack in the future.
@@ -25,9 +27,9 @@ Set up in the cloud configuration at ``/etc/salt/cloud.providers`` or
 
     my-openstack-config:
       # The OpenStack identity service url
-      identity_url: https://region-a.geo-1.identity.hpcloudsvc.com:35357/v2.0/
+      identity_url: https://region-b.geo-1.identity.hpcloudsvc.com:35357/v2.0/
       # The OpenStack compute region
-      compute_region: az-1.region-a.geo-1
+      compute_region: region-b.geo-1
       # The OpenStack compute service name
       compute_name: Compute
       # The OpenStack tenant name (not tenant ID)
@@ -35,7 +37,11 @@ Set up in the cloud configuration at ``/etc/salt/cloud.providers`` or
       # The OpenStack user name
       user: myuser
       # The OpenStack keypair name
-      ssh_key_name
+      ssh_key_name: mykey
+      # Skip SSL certificate validation
+      insecure: false
+      # The ssh key file
+      ssh_key_file: /path/to/keyfile/test.pem
       # The OpenStack network UUIDs
       networks:
           - fixed:
@@ -45,9 +51,12 @@ Set up in the cloud configuration at ``/etc/salt/cloud.providers`` or
       files:
           /path/to/dest.txt:
               /local/path/to/src.txt
-
+      # Skips the service catalog API endpoint, and uses the following
+      base_url: http://192.168.1.101:3000/v2/12345
       provider: openstack
       userdata_file: /tmp/userdata.txt
+      # config_drive is required for userdata at rackspace
+      config_drive: True
 
 For in-house Openstack Essex installation, libcloud needs the service_type :
 
@@ -69,6 +78,15 @@ Either a password or an API key must also be specified:
       # The OpenStack API key
       apikey: 901d3f579h23c8v73q9
 
+Optionally, if you don't want to save plain-text password in your configuration file, you can use keyring:
+
+.. code-block:: yaml
+
+    my-openstack-keyring-config:
+      # The OpenStack password is stored in keyring
+      # don't forget to set the password by running something like:
+      # salt-cloud --set-password=myuser my-openstack-keyring-config
+      password: USE_KEYRING
 
 For local installations that only use private IP address ranges, the
 following option may be useful. Using the old syntax:
@@ -80,9 +98,9 @@ following option may be useful. Using the old syntax:
       ignore_cidr: 192.168.50.0/24
 
 It is possible to upload a small set of files (no more than 5, and nothing too
-large) to the remote server. Generally this should not be needed, as salt itself
-can upload to the server after it is spun up, with nowhere near the same
-restrictions.
+large) to the remote server. Generally this should not be needed, as salt
+itself can upload to the server after it is spun up, with nowhere near the
+same restrictions.
 
 .. code-block:: yaml
 
@@ -90,19 +108,31 @@ restrictions.
       files:
           /path/to/dest.txt:
               /local/path/to/src.txt
+
+Alternatively, one could use the private IP to connect by specifying:
+
+.. code-block:: yaml
+
+    my-openstack-config:
+      ssh_interface: private_ips
+
+
 '''
 
 # The import section is mostly libcloud boilerplate
 
 # Import python libs
 import os
-import copy
 import logging
 import socket
 import pprint
 
 # Import libcloud
-from libcloud.compute.base import NodeState
+try:
+    from libcloud.compute.base import NodeState
+    HAS_LIBCLOUD = True
+except ImportError:
+    HAS_LIBCLOUD = False
 
 # These functions requre libcloud trunk or >= 0.14.0
 HAS014 = False
@@ -121,9 +151,10 @@ import salt.utils
 
 # Import salt.cloud libs
 import salt.utils.cloud
+import salt.utils.pycrypto as sup
 import salt.config as config
 from salt.utils import namespaced_function
-from salt.cloud.exceptions import (
+from salt.exceptions import (
     SaltCloudConfigError,
     SaltCloudNotFound,
     SaltCloudSystemExit,
@@ -157,6 +188,7 @@ list_nodes = namespaced_function(list_nodes, globals())
 list_nodes_full = namespaced_function(list_nodes_full, globals())
 list_nodes_select = namespaced_function(list_nodes_select, globals())
 show_instance = namespaced_function(show_instance, globals())
+get_node = namespaced_function(get_node, globals())
 
 
 # Only load in this module is the OPENSTACK configurations are in place
@@ -164,6 +196,9 @@ def __virtual__():
     '''
     Set up the libcloud functions and check for OPENSTACK configurations
     '''
+    if not HAS_LIBCLOUD:
+        return False
+
     if get_configured_provider() is False:
         return False
 
@@ -200,14 +235,23 @@ def get_conn():
         'ex_tenant_name': config.get_cloud_config_value(
             'tenant', vm_, __opts__, search_global=False
         ),
-   }
+    }
 
-    service_type = config.get_cloud_config_value(
-            'service_type', vm_, __opts__, search_global=False
-        )
+    service_type = config.get_cloud_config_value('service_type',
+                                                 vm_,
+                                                 __opts__,
+                                                 search_global=False)
     if service_type:
         authinfo['ex_force_service_type'] = service_type
 
+    base_url = config.get_cloud_config_value('base_url',
+                                             vm_,
+                                             __opts__,
+                                             search_global=False)
+
+    if base_url:
+        authinfo['ex_force_base_url'] = base_url
+
     insecure = config.get_cloud_config_value(
         'insecure', vm_, __opts__, search_global=False
     )
@@ -215,27 +259,54 @@ def get_conn():
         import libcloud.security
         libcloud.security.VERIFY_SSL_CERT = False
 
+    user = config.get_cloud_config_value(
+        'user', vm_, __opts__, search_global=False
+    )
     password = config.get_cloud_config_value(
         'password', vm_, __opts__, search_global=False
     )
+
     if password is not None:
         authinfo['ex_force_auth_version'] = '2.0_password'
         log.debug('OpenStack authenticating using password')
+        if password == 'USE_KEYRING':
+            # retrieve password from system keyring
+            credential_id = "salt.cloud.provider.{0}".format(__active_provider_name__)
+            logging.debug("Retrieving keyring password for {0} ({1})".format(
+                credential_id,
+                user)
+            )
+            # attempt to retrieve driver specific password first
+            driver_password = salt.utils.cloud.retrieve_password_from_keyring(
+                credential_id,
+                user
+            )
+            if driver_password is None:
+                provider_password = salt.utils.cloud.retrieve_password_from_keyring(
+                    credential_id.split(':')[0],  # fallback to provider level
+                    user)
+                if provider_password is None:
+                    raise SaltCloudSystemExit(
+                        "Unable to retrieve password from keyring for provider {0}".format(
+                            __active_provider_name__
+                        )
+                    )
+                else:
+                    actual_password = provider_password
+        else:
+            actual_password = password
         return driver(
-            config.get_cloud_config_value(
-                'user', vm_, __opts__, search_global=False
-            ),
-            password,
+            user,
+            actual_password,
             **authinfo
         )
 
     authinfo['ex_force_auth_version'] = '2.0_apikey'
     log.debug('OpenStack authenticating using apikey')
     return driver(
-        config.get_cloud_config_value('user', vm_, __opts__, search_global=False),
-        config.get_cloud_config_value('apikey', vm_, __opts__, search_global=False),
-        **authinfo
-    )
+        user,
+        config.get_cloud_config_value('apikey', vm_, __opts__,
+                                      search_global=False), **authinfo)
 
 
 def preferred_ip(vm_, ips):
@@ -313,43 +384,19 @@ def managedcloud(vm_):
     )
 
 
-def create(vm_):
+def request_instance(vm_=None, call=None):
     '''
-    Create a single VM from a data dict
-    '''
-    deploy = config.get_cloud_config_value('deploy', vm_, __opts__)
-    key_filename = config.get_cloud_config_value(
-        'ssh_key_file', vm_, __opts__, search_global=False, default=None
-    )
-    if key_filename is not None:
-        key_filename = os.path.expanduser(key_filename)
-        if not os.path.isfile(key_filename):
-            raise SaltCloudConfigError(
-                'The defined ssh_key_file {0!r} does not exist'.format(
-                    key_filename
-                )
-            )
+    Put together all of the information necessary to request an instance on Openstack
+    and then fire off the request the instance.
 
-    if deploy is True and key_filename is None and \
-            salt.utils.which('sshpass') is None:
+    Returns data about the instance
+    '''
+    if call == 'function':
+        # Technically this function may be called other ways too, but it
+        # definitely cannot be called with --function.
         raise SaltCloudSystemExit(
-            'Cannot deploy salt in a VM if the \'ssh_key_file\' setting '
-            'is not set and \'sshpass\' binary is not present on the '
-            'system for the password.'
+            'The request_instance action must be called with -a or --action.'
         )
-
-    salt.utils.cloud.fire_event(
-        'event',
-        'starting create',
-        'salt/cloud/{0}/creating'.format(vm_['name']),
-        {
-            'name': vm_['name'],
-            'profile': vm_['profile'],
-            'provider': vm_['provider'],
-        },
-    )
-
-    log.info('Creating Cloud VM {0}'.format(vm_['name']))
     salt.utils.cloud.check_name(vm_['name'], 'a-zA-Z0-9._-')
     conn = get_conn()
     kwargs = {
@@ -359,28 +406,29 @@ def create(vm_):
     try:
         kwargs['image'] = get_image(conn, vm_)
     except Exception as exc:
-        log.error(
+        raise SaltCloudSystemExit(
             'Error creating {0} on OPENSTACK\n\n'
             'Could not find image {1}: {2}\n'.format(
                 vm_['name'], vm_['image'], exc
-            ),
-            # Show the traceback if the debug logging level is enabled
-            exc_info=log.isEnabledFor(logging.DEBUG)
+            )
         )
-        return False
 
     try:
         kwargs['size'] = get_size(conn, vm_)
     except Exception as exc:
-        log.error(
+        raise SaltCloudSystemExit(
             'Error creating {0} on OPENSTACK\n\n'
             'Could not find size {1}: {2}\n'.format(
                 vm_['name'], vm_['size'], exc
-            ),
-            # Show the traceback if the debug logging level is enabled
-            exc_info=log.isEnabledFor(logging.DEBUG)
+            )
         )
-        return False
+
+    # Note: This currently requires libcloud trunk
+    avz = config.get_cloud_config_value(
+        'availability_zone', vm_, __opts__, default=None, search_global=False
+    )
+    if avz is not None:
+        kwargs['ex_availability_zone'] = avz
 
     kwargs['ex_keyname'] = config.get_cloud_config_value(
         'ssh_key_name', vm_, __opts__, search_global=False
@@ -412,29 +460,61 @@ def create(vm_):
 
     floating = []
 
-    if HAS014 and networks is not None:
-        for net in networks:
-            if 'fixed' in net:
-                kwargs['networks'] = [
-                    OpenStackNetwork(n, None, None, None) for n in net['fixed']
-                ]
-            elif 'floating' in net:
+    if HAS014:
+        if networks is not None:
+            for net in networks:
+                if 'fixed' in net:
+                    kwargs['networks'] = [
+                        OpenStackNetwork(n, None, None, None)
+                        for n in net['fixed']
+                    ]
+                elif 'floating' in net:
+                    pool = OpenStack_1_1_FloatingIpPool(
+                        net['floating'], conn.connection
+                    )
+                    for idx in pool.list_floating_ips():
+                        if idx.node_id is None:
+                            floating.append(idx)
+                    if not floating:
+                        # Note(pabelanger): We have no available floating IPs.
+                        # For now, we raise an exception and exit.
+                        # A future enhancement might be to allow salt-cloud
+                        # to dynamically allocate new address but that might
+                        raise SaltCloudSystemExit(
+                            'Floating pool {0!r} does not have any more '
+                            'please create some more or use a different '
+                            'pool.'.format(net['floating'])
+                        )
+        # otherwise, attempt to obtain list without specifying pool
+        # this is the same as 'nova floating-ip-list'
+        elif ssh_interface(vm_) != 'private_ips':
+            try:
+                # This try/except is here because it appears some
+                # *cough* Rackspace *cough*
+                # OpenStack providers return a 404 Not Found for the
+                # floating ip pool URL if there are no pools setup
                 pool = OpenStack_1_1_FloatingIpPool(
-                    net['floating'], conn.connection
+                    '', conn.connection
                 )
                 for idx in pool.list_floating_ips():
                     if idx.node_id is None:
                         floating.append(idx)
                 if not floating:
-                    # Note(pabelanger): We have no available floating IPs. For
-                    # now, we raise an exception and exit. A future enhancement
-                    # might be to allow salt-cloud to dynamically allocate new
-                    # address but that might be tricky to manage.
+                    # Note(pabelanger): We have no available floating IPs.
+                    # For now, we raise an exception and exit.
+                    # A future enhancement might be to allow salt-cloud to
+                    # dynamically allocate new address but that might be
+                    # tricky to manage.
                     raise SaltCloudSystemExit(
-                        'Floating pool {0!r} has not more address available, '
-                        'please create some more or use a different '
-                        'pool.'.format(net['floating'])
+                        'There are no more floating IP addresses '
+                        'available, please create some more'
                     )
+            except Exception as e:
+                if str(e).startswith('404'):
+                    pass
+                else:
+                    raise
+    vm_['floating'] = floating
 
     files = config.get_cloud_config_value(
         'files', vm_, __opts__, search_global=False
@@ -453,6 +533,10 @@ def create(vm_):
         with salt.utils.fopen(userdata_file, 'r') as fp:
             kwargs['ex_userdata'] = fp.read()
 
+    kwargs['ex_config_drive'] = config.get_cloud_config_value(
+        'config_drive', vm_, __opts__, search_global=False
+    )
+
     salt.utils.cloud.fire_event(
         'event',
         'requesting instance',
@@ -460,7 +544,8 @@ def create(vm_):
         {'kwargs': {'name': kwargs['name'],
                     'image': kwargs['image'].name,
                     'size': kwargs['size'].name,
-                    'profile': vm_['profile']}}
+                    'profile': vm_['profile']}},
+        transport=__opts__['transport']
     )
 
     default_profile = {}
@@ -471,33 +556,86 @@ def create(vm_):
         'metadata', vm_, __opts__, default=default_profile, search_global=False
     )
     if not isinstance(kwargs['ex_metadata'], dict):
-        raise SaltCloudConfigError(
-                '\'metadata\' should be a dict.'
-        )
+        raise SaltCloudConfigError('\'metadata\' should be a dict.')
 
     try:
         data = conn.create_node(**kwargs)
     except Exception as exc:
-        log.error(
+        raise SaltCloudSystemExit(
             'Error creating {0} on OpenStack\n\n'
             'The following exception was thrown by libcloud when trying to '
             'run the initial deployment: {1}\n'.format(
                 vm_['name'], exc
-            ),
-            # Show the traceback if the debug logging level is enabled
-            exc_info=log.isEnabledFor(logging.DEBUG)
+            )
         )
-        return False
+
+    vm_['password'] = data.extra.get('password', None)
+    return data, vm_
+
+
+def create(vm_):
+    '''
+    Create a single VM from a data dict
+    '''
+    deploy = config.get_cloud_config_value('deploy', vm_, __opts__)
+    key_filename = config.get_cloud_config_value(
+        'ssh_key_file', vm_, __opts__, search_global=False, default=None
+    )
+    if key_filename is not None:
+        key_filename = os.path.expanduser(key_filename)
+        if not os.path.isfile(key_filename):
+            raise SaltCloudConfigError(
+                'The defined ssh_key_file {0!r} does not exist'.format(
+                    key_filename
+                )
+            )
+
+    vm_['key_filename'] = key_filename
+
+    salt.utils.cloud.fire_event(
+        'event',
+        'starting create',
+        'salt/cloud/{0}/creating'.format(vm_['name']),
+        {
+            'name': vm_['name'],
+            'profile': vm_['profile'],
+            'provider': vm_['provider'],
+        },
+        transport=__opts__['transport']
+    )
+
+    conn = get_conn()
+
+    if 'instance_id' in vm_:
+        # This was probably created via another process, and doesn't have
+        # things like salt keys created yet, so let's create them now.
+        if 'pub_key' not in vm_ and 'priv_key' not in vm_:
+            log.debug('Generating minion keys for {0[name]!r}'.format(vm_))
+            vm_['priv_key'], vm_['pub_key'] = salt.utils.cloud.gen_keys(
+                salt.config.get_cloud_config_value(
+                    'keysize',
+                    vm_,
+                    __opts__
+                )
+            )
+        if vm_['key_filename'] is None and 'change_password' in __opts__ and __opts__['change_password'] is True:
+            vm_['password'] = sup.secure_password()
+            conn.root_password(vm_['instance_id'], vm_['password'])
+    else:
+        # Put together all of the information required to request the instance,
+        # and then fire off the request for it
+        data, vm_ = request_instance(vm_)
+
+        # Pull the instance ID, valid for both spot and normal instances
+        vm_['instance_id'] = data.id
 
     def __query_node_data(vm_, data, floating):
         try:
-            nodelist = list_nodes_full()
+            node = show_instance(vm_['name'], 'action')
             log.debug(
                 'Loaded node data for {0}:\n{1}'.format(
                     vm_['name'],
-                    pprint.pformat(
-                        nodelist[vm_['name']]
-                    )
+                    pprint.pformat(node)
                 )
             )
         except Exception as err:
@@ -506,19 +644,19 @@ def create(vm_):
                     err
                 ),
                 # Show the traceback if the debug logging level is enabled
-                exc_info=log.isEnabledFor(logging.DEBUG)
+                exc_info_on_loglevel=logging.DEBUG
             )
             # Trigger a failure in the wait for IP function
             return False
 
-        running = nodelist[vm_['name']]['state'] == NodeState.RUNNING
+        running = node['state'] == NodeState.RUNNING
         if not running:
             # Still not running, trigger another iteration
             return
 
         if rackconnect(vm_) is True:
-            check_libcloud_version('0.14.0', why='rackconnect: True')
-            extra = nodelist[vm_['name']].get('extra')
+            check_libcloud_version((0, 14, 0), why='rackconnect: True')
+            extra = node.get('extra')
             rc_status = extra.get('metadata', {}).get(
                 'rackconnect_automation_status', '')
             access_ip = extra.get('access_ip', '')
@@ -528,7 +666,7 @@ def create(vm_):
                 return
 
         if managedcloud(vm_) is True:
-            extra = nodelist[vm_['name']].get('extra')
+            extra = node.get('extra')
             mc_status = extra.get('metadata', {}).get(
                 'rax_service_level_automation', '')
 
@@ -536,6 +674,7 @@ def create(vm_):
                 log.debug('Waiting for managed cloud automation to complete')
                 return
 
+        public = node['public_ips']
         if floating:
             try:
                 name = data.name
@@ -546,6 +685,8 @@ def create(vm_):
                         ip, name
                     )
                 )
+                data.public_ips.append(ip)
+                public = data.public_ips
             except Exception:
                 # Note(pabelanger): Because we loop, we only want to attach the
                 # floating IP address one. So, expect failures if the IP is
@@ -553,8 +694,7 @@ def create(vm_):
                 pass
 
         result = []
-        private = nodelist[vm_['name']]['private_ips']
-        public = nodelist[vm_['name']]['public_ips']
+        private = node['private_ips']
         if private and not public:
             log.warn(
                 'Private IPs returned, but not public... Checking for '
@@ -603,7 +743,7 @@ def create(vm_):
     try:
         data = salt.utils.cloud.wait_for_ip(
             __query_node_data,
-            update_args=(vm_, data, floating),
+            update_args=(vm_, data, vm_['floating']),
             timeout=config.get_cloud_config_value(
                 'wait_for_ip_timeout', vm_, __opts__, default=10 * 60),
             interval=config.get_cloud_config_value(
@@ -616,7 +756,7 @@ def create(vm_):
         except SaltCloudSystemExit:
             pass
         finally:
-            raise SaltCloudSystemExit(exc.message)
+            raise SaltCloudSystemExit(str(exc))
 
     log.debug('VM is now running')
 
@@ -631,128 +771,8 @@ def create(vm_):
     if not ip_address:
         raise SaltCloudSystemExit('A valid IP address was not found')
 
-    ssh_username = config.get_cloud_config_value(
-        'ssh_username', vm_, __opts__, default='root'
-    )
-    deploy_kwargs = {
-        'host': ip_address,
-        'name': vm_['name'],
-        'sock_dir': __opts__['sock_dir'],
-        'tmp_dir': config.get_cloud_config_value(
-            'tmp_dir', vm_, __opts__, default='/tmp/.saltcloud'
-        ),
-        'deploy_command': config.get_cloud_config_value(
-            'deploy_command', vm_, __opts__,
-            default='/tmp/.saltcloud/deploy.sh',
-        ),
-        'start_action': __opts__['start_action'],
-        'parallel': __opts__['parallel'],
-        'minion_pem': vm_['priv_key'],
-        'minion_pub': vm_['pub_key'],
-        'keep_tmp': __opts__['keep_tmp'],
-        'preseed_minion_keys': vm_.get('preseed_minion_keys', None),
-        'sudo': config.get_cloud_config_value(
-            'sudo', vm_, __opts__, default=(ssh_username != 'root')
-        ),
-        'sudo_password': config.get_cloud_config_value(
-            'sudo_password', vm_, __opts__, default=None
-        ),
-        'display_ssh_output': config.get_cloud_config_value(
-            'display_ssh_output', vm_, __opts__, default=True
-        ),
-        'script_args': config.get_cloud_config_value(
-            'script_args', vm_, __opts__
-        ),
-        'script_env': config.get_cloud_config_value('script_env', vm_, __opts__),
-        'minion_conf': salt.utils.cloud.minion_config(__opts__, vm_)
-    }
-
-    if ssh_username != 'root':
-        deploy_kwargs['username'] = ssh_username
-        deploy_kwargs['tty'] = True
-
-    log.debug('Using {0} as SSH username'.format(ssh_username))
-
-    if key_filename is not None:
-        deploy_kwargs['key_filename'] = key_filename
-        log.debug(
-            'Using {0} as SSH key file'.format(key_filename)
-        )
-    elif hasattr(data, 'extra') and 'password' in data.extra:
-        deploy_kwargs['password'] = data.extra['password']
-        log.debug('Logging into SSH using password')
-
-    ret = {}
-    sudo = config.get_cloud_config_value(
-        'sudo', vm_, __opts__, default=(ssh_username != 'root')
-    )
-    if sudo is not None:
-        deploy_kwargs['sudo'] = sudo
-        log.debug('Running root commands using sudo')
-
-    if config.get_cloud_config_value('deploy', vm_, __opts__) is True:
-        deploy_script = script(vm_)
-        deploy_kwargs['script'] = deploy_script.script
-
-        # Deploy salt-master files, if necessary
-        if config.get_cloud_config_value('make_master', vm_, __opts__) is True:
-            deploy_kwargs['make_master'] = True
-            deploy_kwargs['master_pub'] = vm_['master_pub']
-            deploy_kwargs['master_pem'] = vm_['master_pem']
-            master_conf = salt.utils.cloud.master_config(__opts__, vm_)
-            deploy_kwargs['master_conf'] = master_conf
-
-            if master_conf.get('syndic_master', None):
-                deploy_kwargs['make_syndic'] = True
-
-        deploy_kwargs['make_minion'] = config.get_cloud_config_value(
-            'make_minion', vm_, __opts__, default=True
-        )
-
-        # Check for Windows install params
-        win_installer = config.get_cloud_config_value('win_installer', vm_, __opts__)
-        if win_installer:
-            deploy_kwargs['win_installer'] = win_installer
-            minion = salt.utils.cloud.minion_config(__opts__, vm_)
-            deploy_kwargs['master'] = minion['master']
-            deploy_kwargs['username'] = config.get_cloud_config_value(
-                'win_username', vm_, __opts__, default='Administrator'
-            )
-            deploy_kwargs['password'] = config.get_cloud_config_value(
-                'win_password', vm_, __opts__, default=''
-            )
-
-        # Store what was used to the deploy the VM
-        event_kwargs = copy.deepcopy(deploy_kwargs)
-        del event_kwargs['minion_pem']
-        del event_kwargs['minion_pub']
-        del event_kwargs['sudo_password']
-        if 'password' in event_kwargs:
-            del event_kwargs['password']
-        ret['deploy_kwargs'] = event_kwargs
-
-        salt.utils.cloud.fire_event(
-            'event',
-            'executing deploy script',
-            'salt/cloud/{0}/deploying'.format(vm_['name']),
-            {'kwargs': event_kwargs},
-        )
-
-        deployed = False
-        if win_installer:
-            deployed = salt.utils.cloud.deploy_windows(**deploy_kwargs)
-        else:
-            deployed = salt.utils.cloud.deploy_script(**deploy_kwargs)
-
-        if deployed:
-            log.info('Salt installed on {0}'.format(vm_['name']))
-        else:
-            log.error(
-                'Failed to deploy and start Salt on Cloud VM {0}'.format(
-                    vm_['name']
-                )
-            )
-
+    vm_['ssh_host'] = ip_address
+    ret = salt.utils.cloud.bootstrap(vm_, __opts__)
     ret.update(data.__dict__)
 
     if hasattr(data, 'extra') and 'password' in data.extra:
@@ -774,6 +794,7 @@ def create(vm_):
             'profile': vm_['profile'],
             'provider': vm_['provider'],
         },
+        transport=__opts__['transport']
     )
 
     return ret
