@@ -3,14 +3,17 @@
 A dead simple module wrapping calls to the Chocolatey package manager
 (http://chocolatey.org)
 
-.. versionadded:: 2014.1.0 (Hydrogen)
+.. versionadded:: 2014.1.0
 '''
 
+# Import python libs
 import logging
 import os.path
 import re
 import tempfile
+from distutils.version import LooseVersion as _LooseVersion  # pylint: disable=import-error,no-name-in-module
 
+# Import salt libs
 import salt.utils
 from salt.exceptions import CommandExecutionError, CommandNotFoundError
 
@@ -42,14 +45,52 @@ def _find_chocolatey():
     '''
     Returns the full path to chocolatey.bat on the host.
     '''
-    choc_default = 'C:\\Chocolatey\\bin\\chocolatey.bat'
+    try:
+        return __context__['chocolatey._path']
+    except KeyError:
+        choc_defaults = ['C:\\Chocolatey\\bin\\chocolatey.bat',
+                         'C:\\ProgramData\\Chocolatey\\bin\\chocolatey.exe', ]
 
-    choc_path = __salt__['cmd.which']('chocolatey.bat')
-    if not choc_path:
-        if __salt__['cmd.has_exec'](choc_default):
-            choc_path = choc_default
+        choc_path = __salt__['cmd.which']('chocolatey.exe')
+        if not choc_path:
+            for choc_dir in choc_defaults:
+                if __salt__['cmd.has_exec'](choc_dir):
+                    choc_path = choc_dir
+        if not choc_path:
+            err = ('Chocolatey not installed. Use chocolatey.bootstrap to '
+                   'install the Chocolatey package manager.')
+            log.error(err)
+            raise CommandExecutionError(err)
+        __context__['chocolatey._path'] = choc_path
+        return choc_path
 
-    return choc_path
+
+def chocolatey_version():
+    '''
+    .. versionadded:: 2014.7.0
+
+    Returns the version of Chocolatey installed on the minion.
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' chocolatey.chocolatey_version
+    '''
+    try:
+        return __context__['chocolatey._version']
+    except KeyError:
+        cmd = [_find_chocolatey(), 'help']
+        out = __salt__['cmd.run'](cmd, python_shell=False)
+        for line in out.splitlines():
+            if line.lower().startswith('version: '):
+                try:
+                    __context__['chocolatey._version'] = \
+                        line.split(None, 1)[-1].strip("'")
+                    return __context__['chocolatey._version']
+                except Exception:
+                    pass
+        raise CommandExecutionError('Unable to determine Chocolatey version')
 
 
 def bootstrap(force=False):
@@ -68,13 +109,18 @@ def bootstrap(force=False):
     force
         Run the bootstrap process even if Chocolatey is found in the path.
 
+    CLI Example:
+
     .. code-block:: bash
 
         salt '*' chocolatey.bootstrap
         salt '*' chocolatey.bootstrap force=True
     '''
     # Check if Chocolatey is already present in the path
-    choc_path = _find_chocolatey()
+    try:
+        choc_path = _find_chocolatey()
+    except CommandExecutionError:
+        choc_path = None
     if choc_path and not force:
         return 'Chocolatey found at {0}'.format(choc_path)
 
@@ -82,10 +128,10 @@ def bootstrap(force=False):
     # download required to install PowerShell. That's right, there's more
     # than one! You're welcome.
     ps_downloads = {
-        ('Vista', 'x86'): ('http://download.microsoft.com/download/A/7/5/A75BC017-63CE-47D6-8FA4-AFB5C21BAC54/Windows6.0-KB968930-x86.msu'),
-        ('Vista', 'AMD64'): ('http://download.microsoft.com/download/3/C/8/3C8CF51E-1D9D-4DAA-AAEA-5C48D1CD055C/Windows6.0-KB968930-x64.msu'),
-        ('2008Server', 'x86'): ('http://download.microsoft.com/download/F/9/E/F9EF6ACB-2BA8-4845-9C10-85FC4A69B207/Windows6.0-KB968930-x86.msu'),
-        ('2008Server', 'AMD64'): ('http://download.microsoft.com/download/2/8/6/28686477-3242-4E96-9009-30B16BED89AF/Windows6.0-KB968930-x64.msu')
+        ('Vista', 'x86'): 'http://download.microsoft.com/download/A/7/5/A75BC017-63CE-47D6-8FA4-AFB5C21BAC54/Windows6.0-KB968930-x86.msu',
+        ('Vista', 'AMD64'): 'http://download.microsoft.com/download/3/C/8/3C8CF51E-1D9D-4DAA-AAEA-5C48D1CD055C/Windows6.0-KB968930-x64.msu',
+        ('2008Server', 'x86'): 'http://download.microsoft.com/download/F/9/E/F9EF6ACB-2BA8-4845-9C10-85FC4A69B207/Windows6.0-KB968930-x86.msu',
+        ('2008Server', 'AMD64'): 'http://download.microsoft.com/download/2/8/6/28686477-3242-4E96-9009-30B16BED89AF/Windows6.0-KB968930-x64.msu'
     }
 
     # It took until .NET v4.0 for Microsoft got the hang of making installers,
@@ -104,9 +150,12 @@ def bootstrap(force=False):
             url = ps_downloads[(__grains__['osrelease'], __grains__['cpuarch'])]
             dest = os.path.join(temp_dir, 'powershell.exe')
             __salt__['cp.get_url'](url, dest)
-            result = __salt__['cmd.run_all'](dest + ' /quiet /norestart')
+            cmd = [dest, '/quiet', '/norestart']
+            result = __salt__['cmd.run_all'](cmd, python_shell=False)
             if result['retcode'] != 0:
-                err = 'Installing Windows PowerShell failed. Please run the installer GUI on the host to get a more specific reason.'
+                err = ('Installing Windows PowerShell failed. Please run the '
+                       'installer GUI on the host to get a more specific '
+                       'reason.')
                 log.error(err)
                 raise CommandExecutionError(err)
         else:
@@ -117,20 +166,23 @@ def bootstrap(force=False):
     # Run the .NET Framework 4 web installer
     dest = os.path.join(temp_dir, 'dotnet4.exe')
     __salt__['cp.get_url'](net4_url, dest)
-    result = __salt__['cmd.run_all'](dest + ' /q /norestart')
+    cmd = [dest, '/q', '/norestart']
+    result = __salt__['cmd.run_all'](cmd, python_shell=False)
     if result['retcode'] != 0:
-        err = 'Installing .NET v4.0 failed. Please run the installer GUI on the host to get a more specific reason.'
+        err = ('Installing .NET v4.0 failed. Please run the installer GUI on '
+               'the host to get a more specific reason.')
         log.error(err)
         raise CommandExecutionError(err)
 
     # Run the Chocolatey bootstrap.
-    result = __salt__['cmd.run_all'](
+    cmd = (
         '{0} -NoProfile -ExecutionPolicy unrestricted '
         '-Command "iex ((new-object net.webclient).'
         'DownloadString(\'https://chocolatey.org/install.ps1\'))" '
         '&& SET PATH=%PATH%;%systemdrive%\\chocolatey\\bin'
         .format(ps_path)
     )
+    result = __salt__['cmd.run_all'](cmd, python_shell=True)
 
     if result['retcode'] != 0:
         err = 'Bootstrapping Chocolatey failed: {0}'.format(result['stderr'])
@@ -165,20 +217,15 @@ def list_(filter, all_versions=False, pre_versions=False, source=None):
         salt '*' chocolatey.list <filter> all_versions=True
     '''
     choc_path = _find_chocolatey()
-    if not choc_path:
-        err = 'Chocolatey not installed. Use chocolatey.bootstrap to install the Chocolatey package manager.'
-        log.error(err)
-        raise CommandExecutionError(err)
-
-    cmd = choc_path + ' list ' + filter
+    cmd = [choc_path, 'list', filter]
     if salt.utils.is_true(all_versions):
-        cmd += ' -AllVersions'
+        cmd.append('-AllVersions')
     if salt.utils.is_true(pre_versions):
-        cmd += ' -Prerelease'
+        cmd.append('-Prerelease')
     if source:
-        cmd += ' -Source ' + source
+        cmd.extend(['-Source', source])
 
-    result = __salt__['cmd.run_all'](cmd)
+    result = __salt__['cmd.run_all'](cmd, python_shell=False)
 
     if result['retcode'] != 0:
         err = 'Running chocolatey failed: {0}'.format(result['stderr'])
@@ -195,7 +242,6 @@ def list_(filter, all_versions=False, pre_versions=False, source=None):
                 ret[name] = []
             ret[name].append(ver)
 
-    #log.info(repr(ret))
     return ret
 
 
@@ -210,13 +256,8 @@ def list_webpi():
         salt '*' chocolatey.list_webpi
     '''
     choc_path = _find_chocolatey()
-    if not choc_path:
-        err = 'Chocolatey not installed. Use chocolatey.bootstrap to install the Chocolatey package manager.'
-        log.error(err)
-        raise CommandExecutionError(err)
-
-    cmd = choc_path + ' list -Source webpi'
-    result = __salt__['cmd.run_all'](cmd)
+    cmd = [choc_path, 'list', '-Source', 'webpi']
+    result = __salt__['cmd.run_all'](cmd, python_shell=False)
 
     if result['retcode'] != 0:
         err = 'Running chocolatey failed: {0}'.format(result['stderr'])
@@ -238,13 +279,8 @@ def list_windowsfeatures():
         salt '*' chocolatey.list_windowsfeatures
     '''
     choc_path = _find_chocolatey()
-    if not choc_path:
-        err = 'Chocolatey not installed. Use chocolatey.bootstrap to install the Chocolatey package manager.'
-        log.error(err)
-        raise CommandExecutionError(err)
-
-    cmd = choc_path + ' list -Source windowsfeatures'
-    result = __salt__['cmd.run_all'](cmd)
+    cmd = [choc_path, 'list', '-Source', 'windowsfeatures']
+    result = __salt__['cmd.run_all'](cmd, python_shell=False)
 
     if result['retcode'] != 0:
         err = 'Running chocolatey failed: {0}'.format(result['stderr'])
@@ -279,20 +315,15 @@ def install(name, version=None, source=None, force=False):
         salt '*' chocolatey.install <package name> version=<package version>
     '''
     choc_path = _find_chocolatey()
-    if not choc_path:
-        err = 'Chocolatey not installed. Use chocolatey.bootstrap to install the Chocolatey package manager.'
-        log.error(err)
-        raise CommandExecutionError(err)
-
     # chocolatey helpfully only supports a single package argument
-    cmd = choc_path + ' install ' + name
+    cmd = [choc_path, 'install', name]
     if version:
-        cmd += ' -Version ' + version
+        cmd.extend(['-Version', version])
     if source:
-        cmd += ' -Source ' + source
+        cmd.extend(['-Source', source])
     if salt.utils.is_true(force):
-        cmd += ' -Force'
-    result = __salt__['cmd.run_all'](cmd)
+        cmd.append('-Force')
+    result = __salt__['cmd.run_all'](cmd, python_shell=False)
 
     if result['retcode'] != 0:
         err = 'Running chocolatey failed: {0}'.format(result['stderr'])
@@ -316,13 +347,8 @@ def install_cygwin(name):
         salt '*' chocolatey.install_cygwin <package name>
     '''
     choc_path = _find_chocolatey()
-    if not choc_path:
-        err = 'Chocolatey not installed. Use chocolatey.bootstrap to install the Chocolatey package manager.'
-        log.error(err)
-        raise CommandExecutionError(err)
-
-    cmd = choc_path + ' cygwin ' + name
-    result = __salt__['cmd.run_all'](cmd)
+    cmd = [choc_path, 'cygwin', name]
+    result = __salt__['cmd.run_all'](cmd, python_shell=False)
 
     if result['retcode'] != 0:
         err = 'Running chocolatey failed: {0}'.format(result['stderr'])
@@ -351,15 +377,10 @@ def install_gem(name, version=None):
         salt '*' chocolatey.install_gem <package name> version=<package version>
     '''
     choc_path = _find_chocolatey()
-    if not choc_path:
-        err = 'Chocolatey not installed. Use chocolatey.bootstrap to install the Chocolatey package manager.'
-        log.error(err)
-        raise CommandExecutionError(err)
-
-    cmd = choc_path + ' gem ' + name
+    cmd = [choc_path, 'gem', name]
     if version:
-        cmd += ' -Version ' + version
-    result = __salt__['cmd.run_all'](cmd)
+        cmd.extend(['-Version', version])
+    result = __salt__['cmd.run_all'](cmd, python_shell=False)
 
     if result['retcode'] != 0:
         err = 'Running chocolatey failed: {0}'.format(result['stderr'])
@@ -373,6 +394,12 @@ def install_missing(name, version=None, source=None):
     '''
     Instructs Chocolatey to install a package if it doesn't already exist.
 
+    .. versionchanged:: 2014.7.0
+        If the minion has Chocolatey >= 0.9.8.24 installed, this function calls
+        :mod:`chocolatey.install <salt.modules.chocolatey.install>` instead, as
+        ``installmissing`` is deprecated as of that version and will be removed
+        in Chocolatey 1.0.
+
     name
         The name of the package to be installed. Only accepts a single argument.
 
@@ -392,18 +419,17 @@ def install_missing(name, version=None, source=None):
         salt '*' chocolatey.install_missing <package name> version=<package version>
     '''
     choc_path = _find_chocolatey()
-    if not choc_path:
-        err = 'Chocolatey not installed. Use chocolatey.bootstrap to install the Chocolatey package manager.'
-        log.error(err)
-        raise CommandExecutionError(err)
+    if _LooseVersion(chocolatey_version()) >= _LooseVersion('0.9.8.24'):
+        log.warning('installmissing is deprecated, using install')
+        return install(name, version=version)
 
     # chocolatey helpfully only supports a single package argument
-    cmd = choc_path + ' installmissing ' + name
+    cmd = [choc_path, 'installmissing', name]
     if version:
-        cmd += ' -Version ' + version
+        cmd.extend(['-Version', version])
     if source:
-        cmd += ' -Source ' + source
-    result = __salt__['cmd.run_all'](cmd)
+        cmd.extend(['-Source', source])
+    result = __salt__['cmd.run_all'](cmd, python_shell=False)
 
     if result['retcode'] != 0:
         err = 'Running chocolatey failed: {0}'.format(result['stderr'])
@@ -432,15 +458,10 @@ def install_python(name, version=None):
         salt '*' chocolatey.install_python <package name> version=<package version>
     '''
     choc_path = _find_chocolatey()
-    if not choc_path:
-        err = 'Chocolatey not installed. Use chocolatey.bootstrap to install the Chocolatey package manager.'
-        log.error(err)
-        raise CommandExecutionError(err)
-
-    cmd = choc_path + ' python ' + name
+    cmd = [choc_path, 'python', name]
     if version:
-        cmd += ' -Version ' + version
-    result = __salt__['cmd.run_all'](cmd)
+        cmd.extend(['-Version', version])
+    result = __salt__['cmd.run_all'](cmd, python_shell=False)
 
     if result['retcode'] != 0:
         err = 'Running chocolatey failed: {0}'.format(result['stderr'])
@@ -465,13 +486,8 @@ def install_windowsfeatures(name):
         salt '*' chocolatey.install_windowsfeatures <package name>
     '''
     choc_path = _find_chocolatey()
-    if not choc_path:
-        err = 'Chocolatey not installed. Use chocolatey.bootstrap to install the Chocolatey package manager.'
-        log.error(err)
-        raise CommandExecutionError(err)
-
-    cmd = choc_path + ' windowsfeatures ' + name
-    result = __salt__['cmd.run_all'](cmd)
+    cmd = [choc_path, 'windowsfeatures', name]
+    result = __salt__['cmd.run_all'](cmd, python_shell=False)
 
     if result['retcode'] != 0:
         err = 'Running chocolatey failed: {0}'.format(result['stderr'])
@@ -495,13 +511,8 @@ def install_webpi(name):
         salt '*' chocolatey.install_webpi <package name>
     '''
     choc_path = _find_chocolatey()
-    if not choc_path:
-        err = 'Chocolatey not installed. Use chocolatey.bootstrap to install the Chocolatey package manager.'
-        log.error(err)
-        raise CommandExecutionError(err)
-
-    cmd = choc_path + ' webpi ' + name
-    result = __salt__['cmd.run_all'](cmd)
+    cmd = [choc_path, 'webpi', name]
+    result = __salt__['cmd.run_all'](cmd, python_shell=False)
 
     if result['retcode'] != 0:
         err = 'Running chocolatey failed: {0}'.format(result['stderr'])
@@ -530,16 +541,11 @@ def uninstall(name, version=None):
         salt '*' chocolatey.uninstall <package name> version=<package version>
     '''
     choc_path = _find_chocolatey()
-    if not choc_path:
-        err = 'Chocolatey not installed. Use chocolatey.bootstrap to install the Chocolatey package manager.'
-        log.error(err)
-        raise CommandExecutionError(err)
-
     # chocolatey helpfully only supports a single package argument
-    cmd = choc_path + ' uninstall ' + name
+    cmd = [choc_path, 'uninstall', name]
     if version:
-        cmd += ' -Version ' + version
-    result = __salt__['cmd.run_all'](cmd)
+        cmd.extend(['-Version', version])
+    result = __salt__['cmd.run_all'](cmd, python_shell=False)
 
     if result['retcode'] != 0:
         err = 'Running chocolatey failed: {0}'.format(result['stderr'])
@@ -573,17 +579,12 @@ def update(name, source=None, pre_versions=False):
     '''
     # chocolatey helpfully only supports a single package argument
     choc_path = _find_chocolatey()
-    if not choc_path:
-        err = 'Chocolatey not installed. Use chocolatey.bootstrap to install the Chocolatey package manager.'
-        log.error(err)
-        raise CommandExecutionError(err)
-
-    cmd = choc_path + ' update ' + name
+    cmd = [choc_path, 'update', name]
     if source:
-        cmd += ' -Source ' + source
+        cmd.extend(['-Source', source])
     if salt.utils.is_true(pre_versions):
-        cmd += ' -PreRelease'
-    result = __salt__['cmd.run_all'](cmd)
+        cmd.append('-PreRelease')
+    result = __salt__['cmd.run_all'](cmd, python_shell=False)
 
     if result['retcode'] != 0:
         err = 'Running chocolatey failed: {0}'.format(result['stderr'])
@@ -625,15 +626,15 @@ def version(name, check_remote=False, source=None, pre_versions=False):
         log.error(err)
         raise CommandExecutionError(err)
 
-    cmd = choc_path + ' version ' + name
+    cmd = [choc_path, 'version', name]
     if not salt.utils.is_true(check_remote):
-        cmd += ' -LocalOnly'
+        cmd.append('-LocalOnly')
     if salt.utils.is_true(pre_versions):
-        cmd += ' -Prerelease'
+        cmd.append('-Prerelease')
     if source:
-        cmd += ' -Source ' + source
+        cmd.extend(['-Source', source])
 
-    result = __salt__['cmd.run_all'](cmd)
+    result = __salt__['cmd.run_all'](cmd, python_shell=False)
 
     if result['retcode'] != 0:
         err = 'Running chocolatey failed: {0}'.format(result['stderr'])
