@@ -9,6 +9,12 @@ Manage ruby installations with rbenv.
 import os
 import re
 import logging
+import salt.utils
+import shlex
+try:
+    from shlex import quote as _cmd_quote  # pylint: disable=E0611
+except ImportError:
+    from pipes import quote as _cmd_quote
 
 # Set up logger
 log = logging.getLogger(__name__)
@@ -23,6 +29,33 @@ __opts__ = {
 }
 
 
+def _shlex_split(s):
+    # from python:shlex.split: passing None for s will read
+    # the string to split from standard input.
+    if s is None:
+        ret = shlex.split('')
+    else:
+        ret = shlex.split(s)
+
+    return ret
+
+
+def _parse_env(env):
+    if not env:
+        env = {}
+    if isinstance(env, list):
+        env = salt.utils.repack_dictlist(env)
+    if not isinstance(env, dict):
+        env = {}
+
+    for bad_env_key in (x for x, y in env.iteritems() if y is None):
+        log.error('Environment variable {0!r} passed without a value. '
+                  'Setting value to an empty string'.format(bad_env_key))
+        env[bad_env_key] = ''
+
+    return env
+
+
 def _rbenv_exec(command, args='', env=None, runas=None, ret=None):
     if not is_installed(runas):
         return False
@@ -30,15 +63,15 @@ def _rbenv_exec(command, args='', env=None, runas=None, ret=None):
     binary = _rbenv_bin(runas)
     path = _rbenv_path(runas)
 
-    if env:
-        env = ' {0}'.format(env)
-    env = env or ''
+    environ = _parse_env(env)
+    environ['RBENV_ROOT'] = path
 
-    binary = 'env RBENV_ROOT={0}{1} {2}'.format(path, env, binary)
+    args = ' '.join([_cmd_quote(arg) for arg in _shlex_split(args)])
 
     result = __salt__['cmd.run_all'](
-        '{0} {1} {2}'.format(binary, command, args),
-        runas=runas
+        '{0} {1} {2}'.format(binary, _cmd_quote(command), args),
+        runas=runas,
+        env=environ
     )
 
     if isinstance(ret, dict):
@@ -61,9 +94,10 @@ def _rbenv_path(runas=None):
     if runas in (None, 'root'):
         path = __salt__['config.option']('rbenv.root') or '/usr/local/rbenv'
     else:
-        path = __salt__['config.option']('rbenv.root') or '~{0}/.rbenv'.format(runas)
+        path = (__salt__['config.option']('rbenv.root') or
+                '~{0}/.rbenv'.format(runas))
 
-    return os.path.expanduser(path)
+    return _cmd_quote(os.path.expanduser(path))
 
 
 def _install_rbenv(path, runas=None):
@@ -71,7 +105,8 @@ def _install_rbenv(path, runas=None):
         return True
 
     return 0 == __salt__['cmd.retcode'](
-        'git clone https://github.com/sstephenson/rbenv.git {0}'.format(path), runas=runas)
+        'git clone https://github.com/sstephenson/rbenv.git {0}'
+        .format(_cmd_quote(path)), runas=runas)
 
 
 def _install_ruby_build(path, runas=None):
@@ -80,7 +115,8 @@ def _install_ruby_build(path, runas=None):
         return True
 
     return 0 == __salt__['cmd.retcode'](
-        'git clone https://github.com/sstephenson/ruby-build.git {0}'.format(path), runas=runas)
+        'git clone https://github.com/sstephenson/ruby-build.git {0}'
+        .format(_cmd_quote(path)), runas=runas)
 
 
 def _update_rbenv(path, runas=None):
@@ -88,7 +124,7 @@ def _update_rbenv(path, runas=None):
         return False
 
     return 0 == __salt__['cmd.retcode'](
-        'cd {0} && git pull'.format(path), runas=runas)
+        'git pull', runas=runas, cwd=path)
 
 
 def _update_ruby_build(path, runas=None):
@@ -97,7 +133,7 @@ def _update_ruby_build(path, runas=None):
         return False
 
     return 0 == __salt__['cmd.retcode'](
-        'cd {0} && git pull'.format(path), runas=runas)
+        'git pull', runas=runas, cwd=path)
 
 
 def install(runas=None, path=None):
@@ -152,6 +188,14 @@ def install_ruby(ruby, runas=None):
         The version of Ruby to install, should match one of the
         versions listed by rbenv.list
 
+    Additional environment variables can be configured in pillar /
+    grains / master:
+
+    .. code-block:: yaml
+
+        rbenv:
+          build_env: 'CONFIGURE_OPTS="--no-tcmalloc" CFLAGS="-fno-tree-dce"'
+
     CLI Example:
 
     .. code-block:: bash
@@ -166,7 +210,9 @@ def install_ruby(ruby, runas=None):
     if __grains__['os'] in ('FreeBSD', 'NetBSD', 'OpenBSD'):
         env_list.append('MAKE=gmake')
 
-    if __salt__['config.option']('rbenv.build_env'):
+    if __salt__['config.get']('rbenv:build_env'):
+        env_list.append(__salt__['config.get']('rbenv:build_env'))
+    elif __salt__['config.option']('rbenv.build_env'):
         env_list.append(__salt__['config.option']('rbenv.build_env'))
 
     if env_list:
@@ -288,9 +334,12 @@ def do(cmdline=None, runas=None):
         salt '*' rbenv.do 'gem list bundler' deploy
     '''
     path = _rbenv_path(runas)
+    environ = {'PATH': '{0}/shims:{1}'.format(path, os.environ['PATH'])}
+    cmdline = ' '.join([_cmd_quote(cmd) for cmd in _shlex_split(cmdline)])
     result = __salt__['cmd.run_all'](
-        'env PATH={0}/shims:$PATH {1}'.format(path, cmdline),
-        runas=runas
+        cmdline,
+        runas=runas,
+        env=environ
     )
 
     if result['retcode'] == 0:
