@@ -471,7 +471,7 @@ def get_zone():
 
         salt '*' timezone.get_zone
     '''
-    winzone = __salt__['cmd.run']('tzutil /g')
+    winzone = __salt__['cmd.run'](['tzutil', '/g'], python_shell=False)
     for key in LINTOWIN:
         if LINTOWIN[key] == winzone:
             return key
@@ -490,9 +490,9 @@ def get_offset():
         salt '*' timezone.get_offset
     '''
     string = False
-    zone = __salt__['cmd.run']('tzutil /g')
+    zone = __salt__['cmd.run'](['tzutil', '/g'], python_shell=False)
     prev = ''
-    for line in __salt__['cmd.run']('tzutil /l').splitlines():
+    for line in __salt__['cmd.run'](['tzutil', '/l'], python_shell=False).splitlines():
         if zone == line:
             string = prev
             break
@@ -539,7 +539,8 @@ def set_zone(timezone):
 
         salt '*' timezone.set_zone 'America/Denver'
     '''
-    return __salt__['cmd.retcode']('tzutil /s "{0}"'.format(LINTOWIN[timezone])) == 0
+    cmd = ['tzutil', '/s', LINTOWIN[timezone]]
+    return __salt__['cmd.retcode'](cmd, python_shell=False) == 0
 
 
 def zone_compare(timezone):
@@ -554,7 +555,8 @@ def zone_compare(timezone):
 
         salt '*' timezone.zone_compare 'America/Denver'
     '''
-    return __salt__['cmd.run']('tzutil /g') == LINTOWIN[timezone]
+    cmd = ['tzutil', '/g']
+    return __salt__['cmd.run'](cmd, python_shell=False) == LINTOWIN[timezone]
 
 
 def get_hwclock():
