@@ -23,7 +23,31 @@ no disk space:
       cmd.run:
         - unless: echo 'foo' > /tmp/.test
 
-Note that when executing a command or script, the state (i.e., changed or not)
+Only run if the file specified by ``creates`` does not exist, in this case
+touch /tmp/foo if it does not exist.
+
+.. code-block:: yaml
+
+    touch /tmp/foo:
+      cmd.run:
+        - creates: /tmp/foo
+
+.. note::
+
+    The ``creates`` option was added to version 2014.7.0
+
+Salt determines whether the ``cmd`` state is successfully enforced based on the exit
+code returned by the command. If the command returns a zero exit code, then salt
+determines that the state was successfully enforced. If the script returns a non-zero
+exit code, then salt determines that it failed to successfully enforce the state.
+If a command returns a non-zero exit code but you wish to treat this as a success,
+then you must place the command in a script and explicitly set the exit code of
+the script to zero.
+
+Please note that the success or failure of the state is not affected by whether a state
+change occurred nor the stateful argument.
+
+When executing a command or script, the state (i.e., changed or not)
 of the command is unknown to Salt's state system. Therefore, by default, the
 ``cmd`` state assumes that any command execution results in a changed state.
 
@@ -58,30 +82,34 @@ a simple protocol described below:
    state.`
 
    Here's an example of how one might write a shell script for use with a
-   stateful command::
+   stateful command:
 
-    #!/bin/bash
-    #
-    echo "Working hard..."
+   .. code-block:: bash
 
-    # writing the state line
-    echo  # an empty line here so the next line will be the last.
-    echo "changed=yes comment='something has changed' whatever=123"
+       #!/bin/bash
+       #
+       echo "Working hard..."
 
-   And an example SLS file using this module::
+       # writing the state line
+       echo  # an empty line here so the next line will be the last.
+       echo "changed=yes comment='something has changed' whatever=123"
 
-    Run myscript:
-      cmd.run:
-        - name: /path/to/myscript
-        - cwd: /
-        - stateful: True
+   And an example SLS file using this module:
 
-    Run only if myscript changed something:
-      cmd.wait:
-        - name: echo hello
-        - cwd: /
-        - watch:
-            - cmd: Run myscript
+   .. code-block:: yaml
+
+       Run myscript:
+         cmd.run:
+           - name: /path/to/myscript
+           - cwd: /
+           - stateful: True
+
+       Run only if myscript changed something:
+         cmd.wait:
+           - name: echo hello
+           - cwd: /
+           - watch:
+               - cmd: Run myscript
 
    Note that if the ``cmd.wait`` state also specifies ``stateful: True`` it can
    then be watched by some other states as well.
@@ -104,8 +132,9 @@ it can also watch a git state for changes
           - git: my-project
 
 
-Should I use :mod:`cmd.run <salt.states.cmd.run>` or :mod:`cmd.wait <salt.states.cmd.wait>`?
---------------------------------------------------------------------------------------------
+Should I use :mod:`cmd.run <salt.states.cmd.run>` or :mod:`cmd.wait
+<salt.states.cmd.wait>`?
+-------------------------------------------------------------------------------
 
 These two states are often confused. The important thing to remember about them
 is that :mod:`cmd.run <salt.states.cmd.run>` states are run each time the SLS
@@ -118,19 +147,32 @@ executed when the state it is watching changes. Example:
 .. code-block:: yaml
 
     /usr/local/bin/postinstall.sh:
-      cmd:
-        - wait
+      cmd.wait:
         - watch:
           - pkg: mycustompkg
-      file:
-        - managed
+      file.managed:
         - source: salt://utils/scripts/postinstall.sh
 
     mycustompkg:
-      pkg:
-        - installed
+      pkg.installed:
         - require:
           - file: /usr/local/bin/postinstall.sh
+
+How do I create an environment from a pillar map?
+-------------------------------------------------------------------------------
+
+The map that comes from a pillar cannot be directly consumed by the env option.
+To use it one must convert it to a list. Example:
+
+.. code-block:: yaml
+
+    printenv:
+      cmd.run:
+        - env:
+          {% for key, value in pillar['keys'].iteritems() %}
+          - '{{ key }}': '{{ value }}'
+          {% endfor %}
+
 '''
 
 # Import python libs
@@ -146,11 +188,9 @@ import copy
 import json
 import shlex
 import logging
-import yaml
 
 # Import salt libs
-import salt.utils
-from salt.exceptions import CommandExecutionError
+from salt.exceptions import CommandExecutionError, SaltRenderError
 from salt._compat import string_types
 
 log = logging.getLogger(__name__)
@@ -158,7 +198,7 @@ log = logging.getLogger(__name__)
 
 def _reinterpreted_state(state):
     '''
-    Re-interpret the state returned by salt.sate.run using our protocol.
+    Re-interpret the state returned by salt.state.run using our protocol.
     '''
     ret = state['changes']
     state['changes'] = {}
@@ -179,7 +219,7 @@ def _reinterpreted_state(state):
                 'script JSON output must be a JSON object (e.g., {})!'
             )
         is_json = True
-    except Exception:
+    except ValueError:
         idx = out.rstrip().rfind('\n')
         if idx != -1:
             out = out[idx + 1:]
@@ -230,7 +270,7 @@ def _is_true(val):
     raise ValueError('Failed parsing boolean value: {0}'.format(val))
 
 
-def _run_check(cmd_kwargs, onlyif, unless, group):
+def mod_run_check(cmd_kwargs, onlyif, unless, group, creates):
     '''
     Execute the onlyif and unless logic.
     Return a result dict if:
@@ -239,6 +279,10 @@ def _run_check(cmd_kwargs, onlyif, unless, group):
     * unless succeeded (unless == 0)
     else return True
     '''
+    # never use VT for onlyif/unless executions because this will lead
+    # to quote problems
+    cmd_kwargs = copy.deepcopy(cmd_kwargs)
+    cmd_kwargs['use_vt'] = False
     if group and HAS_GRP:
         try:
             egid = grp.getgrnam(group).gr_gid
@@ -249,24 +293,60 @@ def _run_check(cmd_kwargs, onlyif, unless, group):
                     'result': False}
 
     if onlyif is not None:
-        if not isinstance(onlyif, string_types):
-            if not onlyif:
+        if isinstance(onlyif, string_types):
+            cmd = __salt__['cmd.retcode'](onlyif, ignore_retcode=True, python_shell=True, **cmd_kwargs)
+            log.debug('Last command return code: {0}'.format(cmd))
+            if cmd != 0:
                 return {'comment': 'onlyif execution failed',
+                        'skip_watch': True,
                         'result': True}
-        elif isinstance(onlyif, string_types):
-            if __salt__['cmd.retcode'](onlyif, **cmd_kwargs) != 0:
+        elif isinstance(onlyif, list):
+            for entry in onlyif:
+                cmd = __salt__['cmd.retcode'](entry, ignore_retcode=True, python_shell=True, **cmd_kwargs)
+                log.debug('Last command return code: {0}'.format(cmd))
+                if cmd != 0:
+                    return {'comment': 'onlyif execution failed',
+                        'skip_watch': True,
+                        'result': True}
+        elif not isinstance(onlyif, string_types):
+            if not onlyif:
+                log.debug('Command not run: onlyif did not evaluate to string_type')
                 return {'comment': 'onlyif execution failed',
+                        'skip_watch': True,
                         'result': True}
 
     if unless is not None:
-        if not isinstance(unless, string_types):
+        if isinstance(unless, string_types):
+            cmd = __salt__['cmd.retcode'](unless, ignore_retcode=True, python_shell=True, **cmd_kwargs)
+            log.debug('Last command return code: {0}'.format(cmd))
+            if cmd == 0:
+                return {'comment': 'unless execution succeeded',
+                        'skip_watch': True,
+                        'result': True}
+        elif isinstance(unless, list):
+            cmd = []
+            for entry in unless:
+                cmd.append(__salt__['cmd.retcode'](entry, ignore_retcode=True, python_shell=True, **cmd_kwargs))
+                log.debug('Last command return code: {0}'.format(cmd))
+                if all([c == 0 for c in cmd]):
+                    return {'comment': 'unless execution succeeded',
+                            'skip_watch': True,
+                            'result': True}
+        elif not isinstance(unless, string_types):
             if unless:
+                log.debug('Command not run: unless did not evaluate to string_type')
                 return {'comment': 'unless execution succeeded',
+                        'skip_watch': True,
                         'result': True}
-        elif isinstance(unless, string_types):
-            if __salt__['cmd.retcode'](unless, **cmd_kwargs) == 0:
-                return {'comment': 'unless execution succeeded',
-                        'result': True}
+
+    if isinstance(creates, string_types) and os.path.exists(creates):
+        return {'comment': '{0} exists'.format(creates),
+                'result': True}
+    elif isinstance(creates, list) and all([
+        os.path.exists(path) for path in creates
+    ]):
+        return {'comment': 'All files in creates exist',
+                'result': True}
 
     # No reason to stop, return True
     return True
@@ -275,12 +355,16 @@ def _run_check(cmd_kwargs, onlyif, unless, group):
 def wait(name,
          onlyif=None,
          unless=None,
+         creates=None,
          cwd=None,
          user=None,
          group=None,
          shell=None,
+         env=(),
          stateful=False,
          umask=None,
+         output_loglevel='debug',
+         use_vt=False,
          **kwargs):
     '''
     Run the given command only if the watch statement calls it
@@ -310,12 +394,47 @@ def wait(name,
     shell
         The shell to use for execution, defaults to /bin/sh
 
+    env
+        A list of environment variables to be set prior to execution.
+        Example:
+
+        .. code-block:: yaml
+
+            salt://scripts/foo.sh:
+              cmd.script:
+                - env:
+                  - BATCH: 'yes'
+
+        .. warning::
+
+            The above illustrates a common PyYAML pitfall, that **yes**,
+            **no**, **on**, **off**, **true**, and **false** are all loaded as
+            boolean ``True`` and ``False`` values, and must be enclosed in
+            quotes to be used as strings. More info on this (and other) PyYAML
+            idiosyncrasies can be found :doc:`here
+            </topics/troubleshooting/yaml_idiosyncrasies>`.
+
     umask
          The umask (in octal) to use when running the command.
 
     stateful
         The command being executed is expected to return data about executing
         a state
+
+    creates
+        Only run if the file specified by ``creates`` does not exist.
+
+        .. versionadded:: 2014.7.0
+
+    output_loglevel
+        Control the loglevel at which the output from the command is logged.
+        Note that the command being run will still be logged (loglevel: DEBUG)
+        regardless, unless ``quiet`` is used for this value.
+
+    use_vt
+        Use VT utils (saltstack) to stream the command output more
+        interactively to the console and the logs.
+        This is experimental.
     '''
     # Ignoring our arguments is intentional.
     return {'name': name,
@@ -324,6 +443,10 @@ def wait(name,
             'comment': ''}
 
 
+# Alias "cmd.watch" to "cmd.wait", as this is a common misconfiguration
+watch = wait
+
+
 def wait_script(name,
                 source=None,
                 template=None,
@@ -336,6 +459,8 @@ def wait_script(name,
                 env=None,
                 stateful=False,
                 umask=None,
+                use_vt=False,
+                output_loglevel='debug',
                 **kwargs):
     '''
     Download a script from a remote source and execute it only if a watch
@@ -378,8 +503,24 @@ def wait_script(name,
         The shell to use for execution, defaults to the shell grain
 
     env
-        The root directory of the environment for the referencing script. The
-        environments are defined in the master config file.
+        A list of environment variables to be set prior to execution.
+        Example:
+
+        .. code-block:: yaml
+
+            salt://scripts/foo.sh:
+              cmd.script:
+                - env:
+                  - BATCH: 'yes'
+
+        .. warning::
+
+            The above illustrates a common PyYAML pitfall, that **yes**,
+            **no**, **on**, **off**, **true**, and **false** are all loaded as
+            boolean ``True`` and ``False`` values, and must be enclosed in
+            quotes to be used as strings. More info on this (and other) PyYAML
+            idiosyncrasies can be found :doc:`here
+            </topics/troubleshooting/yaml_idiosyncrasies>`.
 
     umask
          The umask (in octal) to use when running the command.
@@ -387,6 +528,17 @@ def wait_script(name,
     stateful
         The command being executed is expected to return data about executing
         a state
+
+    use_vt
+        Use VT utils (saltstack) to stream the command output more
+        interactively to the console and the logs.
+        This is experimental.
+
+     output_loglevel
+        Control the loglevel at which the output from the command is logged.
+        Note that the command being run will still be logged (loglevel: DEBUG)
+        regardless, unless ``quiet`` is used for this value.
+
     '''
     # Ignoring our arguments is intentional.
     return {'name': name,
@@ -398,15 +550,18 @@ def wait_script(name,
 def run(name,
         onlyif=None,
         unless=None,
+        creates=None,
         cwd=None,
         user=None,
         group=None,
         shell=None,
-        env=(),
+        env=None,
         stateful=False,
         umask=None,
+        output_loglevel='debug',
         quiet=False,
         timeout=None,
+        use_vt=False,
         **kwargs):
     '''
     Run a command if certain circumstances are met.  Use ``cmd.wait`` if you
@@ -438,8 +593,24 @@ def run(name,
         The shell to use for execution, defaults to the shell grain
 
     env
-        Pass in a list or dict of environment variables to be applied to the
-        command upon execution
+        A list of environment variables to be set prior to execution.
+        Example:
+
+        .. code-block:: yaml
+
+            salt://scripts/foo.sh:
+              cmd.script:
+                - env:
+                  - BATCH: 'yes'
+
+        .. warning::
+
+            The above illustrates a common PyYAML pitfall, that **yes**,
+            **no**, **on**, **off**, **true**, and **false** are all loaded as
+            boolean ``True`` and ``False`` values, and must be enclosed in
+            quotes to be used as strings. More info on this (and other) PyYAML
+            idiosyncrasies can be found :doc:`here
+            </topics/troubleshooting/yaml_idiosyncrasies>`.
 
     stateful
         The command being executed is expected to return data about executing
@@ -448,13 +619,50 @@ def run(name,
     umask
         The umask (in octal) to use when running the command.
 
+    output_loglevel
+        Control the loglevel at which the output from the command is logged.
+        Note that the command being run will still be logged (loglevel: DEBUG)
+        regardless, unless ``quiet`` is used for this value.
+
     quiet
         The command will be executed quietly, meaning no log entries of the
-        actual command or its return data
+        actual command or its return data. This is deprecated as of the
+        **2014.1.0** release, and is being replaced with
+        ``output_loglevel: quiet``.
 
     timeout
         If the command has not terminated after timeout seconds, send the
         subprocess sigterm, and if sigterm is ignored, follow up with sigkill
+
+    creates
+        Only run if the file specified by ``creates`` does not exist.
+
+        .. versionadded:: 2014.7.0
+
+    use_vt
+        Use VT utils (saltstack) to stream the command output more
+        interactively to the console and the logs.
+        This is experimental.
+
+    .. note::
+
+        cmd.run supports the usage of ``reload_modules``. This functionality
+        allows you to force Salt to reload all modules. You should only use
+        ``reload_modules`` if your cmd.run does some sort of installation
+        (such as ``pip``), if you do not reload the modules future items in
+        your state which rely on the software being installed will fail.
+
+        .. code-block:: yaml
+
+            getpip:
+              cmd.run:
+                - name: /usr/bin/python /usr/local/sbin/get-pip.py
+                - unless: which pip
+                - require:
+                  - pkg: python
+                  - file: /usr/local/sbin/get-pip.py
+                - reload_modules: True
+
     '''
     ### NOTE: The keyword arguments in **kwargs are ignored in this state, but
     ###       cannot be removed from the function definition, otherwise the use
@@ -467,63 +675,33 @@ def run(name,
            'comment': ''}
 
     if cwd and not os.path.isdir(cwd):
-        ret['comment'] = 'Desired working directory is not available'
+        ret['comment'] = (
+            'Desired working directory "{0}" '
+            'is not available'
+        ).format(cwd)
         return ret
 
-    if env:
-        if isinstance(env, basestring):
-            try:
-                env = yaml.safe_load(env)
-            except Exception:
-                _env = {}
-                for var in env.split():
-                    try:
-                        key, val = var.split('=')
-                        _env[key] = val
-                    except ValueError:
-                        ret['comment'] = \
-                            'Invalid environmental var: "{0}"'.format(var)
-                        return ret
-                env = _env
-        elif isinstance(env, dict):
-            pass
-
-        elif isinstance(env, list):
-            _env = {}
-            for comp in env:
-                try:
-                    if isinstance(comp, basestring):
-                        _env.update(yaml.safe_load(comp))
-                    if isinstance(comp, dict):
-                        _env.update(comp)
-                    else:
-                        ret['comment'] = \
-                            'Invalid environmental var: "{0}"'.format(env)
-                        return ret
-                except Exception:
-                    _env = {}
-                    for var in comp.split():
-                        try:
-                            key, val = var.split('=')
-                            _env[key] = val
-                        except ValueError:
-                            ret['comment'] = \
-                                'Invalid environmental var: "{0}"'.format(var)
-                            return ret
-            env = _env
+    # Need the check for None here, if env is not provided then it falls back
+    # to None and it is assumed that the environment is not being overridden.
+    if env is not None and not isinstance(env, (list, dict)):
+        ret['comment'] = ('Invalidly-formatted \'env\' parameter. See '
+                          'documentation.')
+        return ret
 
     if HAS_GRP:
         pgid = os.getegid()
 
     cmd_kwargs = {'cwd': cwd,
                   'runas': user,
+                  'use_vt': use_vt,
                   'shell': shell or __grains__['shell'],
                   'env': env,
                   'umask': umask,
+                  'output_loglevel': output_loglevel,
                   'quiet': quiet}
 
     try:
-        cret = _run_check(cmd_kwargs, onlyif, unless, group)
+        cret = mod_run_check(cmd_kwargs, onlyif, unless, group, creates)
         if isinstance(cret, dict):
             ret.update(cret)
             return ret
@@ -531,7 +709,9 @@ def run(name,
         # Wow, we passed the test, run this sucker!
         if not __opts__['test']:
             try:
-                cmd_all = __salt__['cmd.run_all'](name, timeout=timeout, **cmd_kwargs)
+                cmd_all = __salt__['cmd.run_all'](
+                    name, timeout=timeout, python_shell=True, **cmd_kwargs
+                )
             except CommandExecutionError as err:
                 ret['comment'] = str(err)
                 return ret
@@ -554,6 +734,7 @@ def script(name,
            template=None,
            onlyif=None,
            unless=None,
+           creates=None,
            cwd=None,
            user=None,
            group=None,
@@ -562,51 +743,66 @@ def script(name,
            stateful=False,
            umask=None,
            timeout=None,
-           __env__='base',
+           use_vt=False,
+           output_loglevel='debug',
            **kwargs):
     '''
-    Download a script from a remote source and execute it. The name can be the
-    source or the source value can be defined.
+    Download a script and execute it with specified arguments.
 
     source
-        The source script being downloaded to the minion, this source script is
-        hosted on the salt master server.  If the file is located on the master
-        in the directory named spam, and is called eggs, the source string is
-        salt://spam/eggs
+        The location of the script to download. If the file is located on the
+        master in the directory named spam, and is called eggs, the source
+        string is salt://spam/eggs
 
     template
         If this setting is applied then the named templating engine will be
-        used to render the downloaded file, currently jinja, mako, and wempy
+        used to render the downloaded file. Currently jinja, mako, and wempy
         are supported
 
     name
-        The command to execute, remember that the command will execute with the
-        path and permissions of the salt-minion.
+        Either "cmd arg1 arg2 arg3..." (cmd is not used) or a source
+        "salt://...".
 
     onlyif
-        A command to run as a check, run the named command only if the command
-        passed to the ``onlyif`` option returns true
+        Run the named command only if the command passed to the ``onlyif``
+        option returns true
 
     unless
-        A command to run as a check, only run the named command if the command
-        passed to the ``unless`` option returns false
+        Run the named command only if the command passed to the ``unless``
+        option returns false
 
     cwd
         The current working directory to execute the command in, defaults to
         /root
 
     user
-        The user name to run the command as
+        The name of the user to run the command as
 
     group
         The group context to run the command as
 
     shell
-        The shell to use for execution, defaults to the shell grain
+        The shell to use for execution. The default is set in grains['shell']
 
     env
-        Pass in a list or dict of environment variables to be applied to the
-        command upon execution
+        A list of environment variables to be set prior to execution.
+        Example:
+
+        .. code-block:: yaml
+
+            salt://scripts/foo.sh:
+              cmd.script:
+                - env:
+                  - BATCH: 'yes'
+
+        .. warning::
+
+            The above illustrates a common PyYAML pitfall, that **yes**,
+            **no**, **on**, **off**, **true**, and **false** are all loaded as
+            boolean ``True`` and ``False`` values, and must be enclosed in
+            quotes to be used as strings. More info on this (and other) PyYAML
+            idiosyncrasies can be found :doc:`here
+            </topics/troubleshooting/yaml_idiosyncrasies>`.
 
     umask
          The umask (in octal) to use when running the command.
@@ -621,33 +817,44 @@ def script(name,
 
     args
         String of command line args to pass to the script.  Only used if no
-        args are specified as part of the `name` argument.
+        args are specified as part of the `name` argument. To pass a string
+        containing spaces in YAML, you will need to doubly-quote it:  "arg1
+        'arg two' arg3"
 
-    __env__
-        The root directory of the environment for the referencing script. The
-        environments are defined in the master config file.
+    creates
+        Only run if the file specified by ``creates`` does not exist.
+
+        .. versionadded:: 2014.7.0
+
+    use_vt
+        Use VT utils (saltstack) to stream the command output more
+        interactively to the console and the logs.
+        This is experimental.
+
+    output_loglevel
+        Control the loglevel at which the output from the command is logged.
+        Note that the command being run will still be logged (loglevel: DEBUG)
+        regardless, unless ``quiet`` is used for this value.
 
     '''
-    ret = {'changes': {},
-           'comment': '',
-           'name': name,
-           'result': False}
+    ret = {'name': name,
+           'changes': {},
+           'result': False,
+           'comment': ''}
 
     if cwd and not os.path.isdir(cwd):
-        ret['comment'] = 'Desired working directory is not available'
+        ret['comment'] = (
+            'Desired working directory "{0}" '
+            'is not available'
+        ).format(cwd)
         return ret
 
-    if isinstance(env, string_types):
-        msg = (
-            'Passing a salt environment should be done using \'__env__\' not '
-            '\'env\'. This warning will go away in salt 0.19.0 and this will '
-            'be the default and expected behaviour. Please update your state '
-            'files.'
-        )
-        salt.utils.warn_until((0, 19), msg)
-        ret.setdefault('warnings', []).append(msg)
-        # Backwards compatibility
-        __env__ = env
+    # Need the check for None here, if env is not provided then it falls back
+    # to None and it is assumed that the environment is not being overridden.
+    if env is not None and not isinstance(env, (list, dict)):
+        ret['comment'] = ('Invalidly-formatted \'env\' parameter. See '
+                          'documentation.')
+        return ret
 
     if HAS_GRP:
         pgid = os.getegid()
@@ -664,7 +871,9 @@ def script(name,
                        'template': template,
                        'umask': umask,
                        'timeout': timeout,
-                       '__env__': __env__})
+                       'output_loglevel': output_loglevel,
+                       'use_vt': use_vt,
+                       'saltenv': __env__})
 
     run_check_cmd_kwargs = {
         'cwd': cwd,
@@ -681,8 +890,8 @@ def script(name,
         cmd_kwargs.update({'args': name.split(' ', 1)[1]})
 
     try:
-        cret = _run_check(
-            run_check_cmd_kwargs, onlyif, unless, group
+        cret = mod_run_check(
+            run_check_cmd_kwargs, onlyif, unless, group, creates
         )
         if isinstance(cret, dict):
             ret.update(cret)
@@ -690,14 +899,14 @@ def script(name,
 
         if __opts__['test']:
             ret['result'] = None
-            ret['comment'] = 'Command "{0}" would have been executed'
+            ret['comment'] = 'Command {0!r} would have been executed'
             ret['comment'] = ret['comment'].format(name)
             return _reinterpreted_state(ret) if stateful else ret
 
         # Wow, we passed the test, run this sucker!
         try:
-            cmd_all = __salt__['cmd.script'](source, **cmd_kwargs)
-        except (CommandExecutionError, IOError) as err:
+            cmd_all = __salt__['cmd.script'](source, python_shell=True, **cmd_kwargs)
+        except (CommandExecutionError, SaltRenderError, IOError) as err:
             ret['comment'] = str(err)
             return ret
 
@@ -707,10 +916,10 @@ def script(name,
         else:
             ret['result'] = not bool(cmd_all['retcode'])
         if ret.get('changes', {}).get('cache_error'):
-            ret['comment'] = 'Unable to cache script {0} from env ' \
-                             '\'{1}\''.format(source, env)
+            ret['comment'] = 'Unable to cache script {0} from saltenv ' \
+                             '{1!r}'.format(source, __env__)
         else:
-            ret['comment'] = 'Command "{0}" run'.format(name)
+            ret['comment'] = 'Command {0!r} run'.format(name)
         return _reinterpreted_state(ret) if stateful else ret
 
     finally:
@@ -724,19 +933,23 @@ def call(name,
          kws=None,
          onlyif=None,
          unless=None,
+         creates=None,
+         output_loglevel='debug',
+         use_vt=False,
          **kwargs):
     '''
     Invoke a pre-defined Python function with arguments specified in the state
     declaration. This function is mainly used by the
     :mod:`salt.renderers.pydsl` renderer.
 
-    The interpretation of `onlyif` and `unless` arguments are identical to
-    those of :func:`salt.states.cmd.run`, and all other arguments(`cwd`,
-    `runas`, ...) allowed by `cmd.run` are allowed here, except that their
-    effects apply only to the commands specified in `onlyif` and `unless`
-    rather than to the function to be invoked.
+    The interpretation of ``onlyif`` and ``unless`` arguments are identical to
+    those of :mod:`cmd.run <salt.states.cmd.run>`, and all other
+    arguments(``cwd``, ``runas``, ...) allowed by :mod:`cmd.run
+    <salt.states.cmd.run>` are allowed here, except that their effects apply
+    only to the commands specified in `onlyif` and `unless` rather than to the
+    function to be invoked.
 
-    In addition the `stateful` argument has no effects here.
+    In addition, the ``stateful`` argument has no effects here.
 
     The return value of the invoked function will be interpreted as follows.
 
@@ -744,14 +957,16 @@ def call(name,
     which expects it to have the usual structure returned by any salt state
     function.
 
-    Otherwise, the return value(denoted as ``result`` in the code below) is
+    Otherwise, the return value (denoted as ``result`` in the code below) is
     expected to be a JSON serializable object, and this dictionary is returned:
 
     .. code-block:: python
 
-        { 'changes': { 'retval': result },
-          'result': True if result is None else bool(result),
-          'comment': result if isinstance(result, basestring) else ''
+        {
+            'name': name
+            'changes': {'retval': result},
+            'result': True if result is None else bool(result),
+            'comment': result if isinstance(result, string_types) else ''
         }
     '''
     ret = {'name': name,
@@ -763,11 +978,13 @@ def call(name,
                   'runas': kwargs.get('user'),
                   'shell': kwargs.get('shell') or __grains__['shell'],
                   'env': kwargs.get('env'),
+                  'use_vt': use_vt,
+                  'output_loglevel': output_loglevel,
                   'umask': kwargs.get('umask')}
     if HAS_GRP:
         pgid = os.getegid()
     try:
-        cret = _run_check(cmd_kwargs, onlyif, unless, None)
+        cret = mod_run_check(cmd_kwargs, onlyif, unless, None, creates)
         if isinstance(cret, dict):
             ret.update(cret)
             return ret
@@ -784,7 +1001,7 @@ def call(name,
         # result must be JSON serializable else we get an error
         ret['changes'] = {'retval': result}
         ret['result'] = True if result is None else bool(result)
-        if isinstance(result, basestring):
+        if isinstance(result, string_types):
             ret['comment'] = result
         return ret
 
@@ -795,7 +1012,10 @@ def wait_call(name,
               kws=None,
               onlyif=None,
               unless=None,
+              creates=None,
               stateful=False,
+              use_vt=False,
+              output_loglevel='debug',
               **kwargs):
     # Ignoring our arguments is intentional.
     return {'name': name,
@@ -808,7 +1028,7 @@ def mod_watch(name, **kwargs):
     '''
     Execute a cmd function based on a watch call
     '''
-    if kwargs['sfun'] == 'wait' or kwargs['sfun'] == 'run':
+    if kwargs['sfun'] in ('wait', 'run', 'watch'):
         if kwargs.get('stateful'):
             kwargs.pop('stateful')
             return _reinterpreted_state(run(name, **kwargs))
@@ -827,7 +1047,9 @@ def mod_watch(name, **kwargs):
         else:
             return {'name': name,
                     'changes': {},
-                    'comment': 'cmd.{0[sfun]} needs a named parameter func'.format(kwargs),
+                    'comment': (
+                        'cmd.{0[sfun]} needs a named parameter func'
+                    ).format(kwargs),
                     'result': False}
 
     return {'name': name,
