@@ -38,11 +38,11 @@ When the tag is exactly 20 characters no padded is done.
 The get_event method intelligently figures out if the tag is longer than 20 characters.
 
 
-The convention for namespacing is to use dot characters "." as the name space delimeter.
+The convention for namespacing is to use dot characters "." as the name space delimiter.
 The name space "salt" is reserved by SaltStack for internal events.
 
 For example:
-Namspaced tag
+Namespaced tag
     'salt.runner.manage.status.start'
 
 '''
@@ -57,7 +57,6 @@ import logging
 import time
 import datetime
 import multiprocessing
-from multiprocessing import Process
 from collections import MutableMapping
 
 # Import third party libs
@@ -73,6 +72,8 @@ import salt.payload
 import salt.loader
 import salt.state
 import salt.utils
+import salt.utils.cache
+import salt.utils.process
 from salt._compat import string_types
 log = logging.getLogger(__name__)
 
@@ -83,8 +84,8 @@ SUB_EVENT = set([
             'state.sls',
             ])
 
-TAGEND = '\n\n'  # long tag delimeter
-TAGPARTER = '/'  # name spaced tag delimeter
+TAGEND = '\n\n'  # long tag delimiter
+TAGPARTER = '/'  # name spaced tag delimiter
 SALT = 'salt'  # base prefix for all salt/ events
 # dict map of namespaced base tag prefixes for salt events
 TAGS = {
@@ -97,13 +98,30 @@ TAGS = {
     'wheel': 'wheel',  # prefix for all salt/wheel events
     'cloud': 'cloud',  # prefix for all salt/cloud events
     'fileserver': 'fileserver',  # prefix for all salt/fileserver events
+    'queue': 'queue',  # prefix for all salt/queue events
 }
 
 
+def get_event(node, sock_dir=None, transport='zeromq', opts=None, listen=True):
+    '''
+    Return an event object suitable for the named transport
+    '''
+    if transport == 'zeromq':
+        if node == 'master':
+            return MasterEvent(sock_dir or opts.get('sock_dir', None))
+        return SaltEvent(node, sock_dir, opts)
+    elif transport == 'raet':
+        import salt.utils.raetevent
+        return salt.utils.raetevent.SaltEvent(node,
+                                              sock_dir=sock_dir,
+                                              listen=listen,
+                                              opts=opts)
+
+
 def tagify(suffix='', prefix='', base=SALT):
     '''
     convenience function to build a namespaced event tag string
-    from joinning with the TABPART character the base, prefix and suffix
+    from joining with the TABPART character the base, prefix and suffix
 
     If string prefix is a valid key in TAGS Then use the value of key prefix
     Else use prefix string
@@ -124,27 +142,29 @@ class SaltEvent(object):
     '''
     The base class used to manage salt events
     '''
-    def __init__(self, node, sock_dir=None, **kwargs):
+    def __init__(self, node, sock_dir=None, opts=None):
         self.serial = salt.payload.Serial({'serial': 'msgpack'})
         self.context = zmq.Context()
         self.poller = zmq.Poller()
         self.cpub = False
         self.cpush = False
-        self.puburi, self.pulluri = self.__load_uri(sock_dir, node, **kwargs)
+        if opts is None:
+            opts = {}
+        self.opts = opts
+        if sock_dir is None:
+            sock_dir = opts.get('sock_dir', None)
+        self.puburi, self.pulluri = self.__load_uri(sock_dir, node)
         self.pending_events = []
 
-    def __load_uri(self, sock_dir, node, **kwargs):
+    def __load_uri(self, sock_dir, node):
         '''
         Return the string URI for the location of the pull and pub sockets to
         use for firing and listening to events
         '''
-        hash_type = getattr(hashlib, kwargs.get('hash_type', 'md5'))
-        # Substr the first 10 chars off, because some algorithms produce
-        # longer hashes than others, and may exceed the IPC maximum length
-        # for UNIX sockets.
-        id_hash = hash_type(kwargs.get('id', '')).hexdigest()
-        if kwargs.get('hash_type', 'md5') == 'sha256':
-            id_hash = id_hash[:10]
+        hash_type = getattr(hashlib, self.opts.get('hash_type', 'md5'))
+        # Only use the first 10 chars to keep longer hashes from exceeding the
+        # max socket path length.
+        id_hash = hash_type(self.opts.get('id', '')).hexdigest()[:10]
         if node == 'master':
             puburi = 'ipc://{0}'.format(os.path.join(
                     sock_dir,
@@ -157,12 +177,12 @@ class SaltEvent(object):
                     ))
             salt.utils.check_ipc_path_max_len(pulluri)
         else:
-            if kwargs.get('ipc_mode', '') == 'tcp':
+            if self.opts.get('ipc_mode', '') == 'tcp':
                 puburi = 'tcp://127.0.0.1:{0}'.format(
-                        kwargs.get('tcp_pub_port', 4510)
+                        self.opts.get('tcp_pub_port', 4510)
                         )
                 pulluri = 'tcp://127.0.0.1:{0}'.format(
-                        kwargs.get('tcp_pull_port', 4511)
+                        self.opts.get('tcp_pull_port', 4511)
                         )
             else:
                 puburi = 'ipc://{0}'.format(os.path.join(
@@ -215,7 +235,7 @@ class SaltEvent(object):
         '''
         self.push = self.context.socket(zmq.PUSH)
         try:
-            # bug in 0MQ default send timeout of -1 (inifinite) is not infinite
+            # bug in 0MQ default send timeout of -1 (infinite) is not infinite
             self.push.setsockopt(zmq.SNDTIMEO, timeout)
         except AttributeError:
             # This is for ZMQ < 2.2 (Caught when ssh'ing into the Jenkins
@@ -233,58 +253,118 @@ class SaltEvent(object):
             mtag = raw[0:20].rstrip('|')
             mdata = raw[20:]
         else:  # new style
-            mtag, sep, mdata = raw.partition(TAGEND)  # split tag from data
+            mtag, _, mdata = raw.partition(TAGEND)  # split tag from data
 
         data = serial.loads(mdata)
         return mtag, data
 
-    def get_event(self, wait=5, tag='', full=False, use_pending=False):
+    def _check_pending(self, tag, pending_tags):
+        """Check the pending_events list for events that match the tag
+
+        :param tag: The tag to search for
+        :type tag: str
+        :param pending_tags: List of tags to preserve
+        :type pending_tags: list[str]
+        :return:
+        """
+        old_events = self.pending_events
+        self.pending_events = []
+        ret = None
+        for evt in old_events:
+            if evt['tag'].startswith(tag):
+                if ret is None:
+                    ret = evt
+                else:
+                    self.pending_events.append(evt)
+            elif any(evt['tag'].startswith(ptag) for ptag in pending_tags):
+                self.pending_events.append(evt)
+        return ret
+
+    def _get_event(self, wait, tag, pending_tags):
+        start = time.time()
+        timeout_at = start + wait
+        while not wait or time.time() <= timeout_at:
+            # convert to milliseconds
+            socks = dict(self.poller.poll(wait * 1000))
+            if socks.get(self.sub) != zmq.POLLIN:
+                continue
+
+            try:
+                # Please do not use non-blocking mode here.
+                # Reliability is more important than pure speed on the event bus.
+                ret = self.get_event_block()
+            except zmq.ZMQError as ex:
+                if ex.errno == errno.EAGAIN or ex.errno == errno.EINTR:
+                    continue
+                else:
+                    raise
+
+            if not ret['tag'].startswith(tag):  # tag not match
+                if any(ret['tag'].startswith(ptag) for ptag in pending_tags):
+                    self.pending_events.append(ret)
+                if wait:  # only update the wait timeout if we had one
+                    wait = timeout_at - time.time()
+                continue
+
+            log.trace('get_event() received = {0}'.format(ret))
+            return ret
+
+        return None
+
+    def get_event(self, wait=5, tag='', full=False, use_pending=False, pending_tags=None):
         '''
         Get a single publication.
-        IF no publication available THEN block for upto wait seconds
+        IF no publication available THEN block for up to wait seconds
         AND either return publication OR None IF no publication available.
 
         IF wait is 0 then block forever.
 
+        New in Boron always checks the list of pending events
+
         use_pending
             Defines whether to keep all unconsumed events in a pending_events
             list, or to discard events that don't match the requested tag.  If
             set to True, MAY CAUSE MEMORY LEAKS.
+
+        pending_tags
+            Add any events matching the listed tags to the pending queue.
+            Still MAY CAUSE MEMORY LEAKS but less likely than use_pending
+            assuming you later get_event for the tags you've listed here
+
+            New in Boron
         '''
         self.subscribe()
 
+        if pending_tags is None:
+            pending_tags = []
         if use_pending:
-            for evt in [x for x in self.pending_events
-                        if x['tag'].startswith(tag)]:
-                self.pending_events.remove(evt)
-                if full:
-                    return evt
-                else:
-                    return evt['data']
+            pending_tags = ['']
 
-        start = time.time()
-        timeout_at = start + wait
-        while not wait or time.time() <= timeout_at:
-            socks = dict(self.poller.poll(wait * 1000))  # convert to milliseconds
-            if self.sub in socks and socks[self.sub] == zmq.POLLIN:
-                raw = self.sub.recv()
-            else:
-                continue
-            mtag, data = self.unpack(raw, self.serial)
+        ret = self._check_pending(tag, pending_tags)
+        if ret is None:
+            ret = self._get_event(wait, tag, pending_tags)
 
-            ret = {'data': data,
-                    'tag': mtag}
+        if ret is None or full:
+            return ret
+        else:
+            return ret['data']
 
-            if not mtag.startswith(tag):  # tag not match
-                if use_pending:
-                    self.pending_events.append(ret)
-                wait = timeout_at - time.time()
-                continue
+    def get_event_noblock(self):
+        '''Get the raw event without blocking or any other niceties
+        '''
+        if not self.cpub:
+            self.connect_pub()
+        raw = self.sub.recv(zmq.NOBLOCK)
+        mtag, data = self.unpack(raw, self.serial)
+        return {'data': data, 'tag': mtag}
 
-            if full:
-                return ret
-            return data
-        return None
+    def get_event_block(self):
+        '''Get the raw event in a blocking fashion
+           Slower, but decreases the possibility of dropped events
+        '''
+        raw = self.sub.recv()
+        mtag, data = self.unpack(raw, self.serial)
+        return {'data': data, 'tag': mtag}
 
     def iter_events(self, tag='', full=False):
         '''
@@ -298,7 +378,7 @@ class SaltEvent(object):
 
     def fire_event(self, data, tag, timeout=1000):
         '''
-        Send a single event into the publisher with paylod dict "data" and event
+        Send a single event into the publisher with payload dict "data" and event
         identifier "tag"
 
         Supports new style long tags.
@@ -315,15 +395,19 @@ class SaltEvent(object):
         if not self.cpush:
             self.connect_pull(timeout=timeout)
 
-        data['_stamp'] = datetime.datetime.now().isoformat()
+        data['_stamp'] = datetime.datetime.utcnow().isoformat()
 
         tagend = ''
         if len(tag) <= 20:  # old style compatible tag
             tag = '{0:|<20}'.format(tag)  # pad with pipes '|' to 20 character length
         else:  # new style longer than 20 chars
             tagend = TAGEND
-
-        event = '{0}{1}{2}'.format(tag, tagend, self.serial.dumps(data))
+        serialized_data = salt.utils.trim_dict(self.serial.dumps(data),
+                self.opts.get('max_event_size', 1048576),
+                is_msgpacked=True
+                )
+        log.debug('Sending event - data = {0}'.format(data))
+        event = '{0}{1}{2}'.format(tag, tagend, serialized_data)
         try:
             self.push.send(event)
         except Exception as ex:
@@ -409,7 +493,6 @@ class MasterEvent(SaltEvent):
     '''
     def __init__(self, sock_dir):
         super(MasterEvent, self).__init__('master', sock_dir)
-        self.connect_pub()
 
 
 class LocalClientEvent(MasterEvent):
@@ -423,11 +506,11 @@ class MinionEvent(SaltEvent):
     '''
     Create a master event management object
     '''
-    def __init__(self, **kwargs):
-        super(MinionEvent, self).__init__('minion', **kwargs)
+    def __init__(self, opts):
+        super(MinionEvent, self).__init__('minion', sock_dir=opts.get('sock_dir', None), opts=opts)
 
 
-class EventPublisher(Process):
+class EventPublisher(multiprocessing.Process):
     '''
     The interface that takes master events and republishes them out to anyone
     who wants to listen
@@ -440,6 +523,7 @@ class EventPublisher(Process):
         '''
         Bind the pub and pull sockets for events
         '''
+        salt.utils.appendproctitle(self.__class__.__name__)
         linger = 5000
         # Set up the context
         self.context = zmq.Context(1)
@@ -501,7 +585,10 @@ class Reactor(multiprocessing.Process, salt.state.Compiler):
     def __init__(self, opts):
         multiprocessing.Process.__init__(self)
         salt.state.Compiler.__init__(self, opts)
-        self.wrap = ReactWrap(self.opts)
+
+        local_minion_opts = self.opts.copy()
+        local_minion_opts['file_client'] = 'local'
+        self.minion = salt.minion.MasterMinion(local_minion_opts)
 
     def render_reaction(self, glob_ref, tag, data):
         '''
@@ -509,6 +596,10 @@ class Reactor(multiprocessing.Process, salt.state.Compiler):
         the data structure
         '''
         react = {}
+
+        if glob_ref.startswith('salt://'):
+            glob_ref = self.minion.functions['cp.cache_file'](glob_ref)
+
         for fn_ in glob.glob(glob_ref):
             try:
                 react.update(self.render_template(
@@ -526,7 +617,7 @@ class Reactor(multiprocessing.Process, salt.state.Compiler):
         '''
         log.debug('Gathering reactors for tag {0}'.format(tag))
         reactors = []
-        if isinstance(self.opts['reactor'], basestring):
+        if isinstance(self.opts['reactor'], string_types):
             try:
                 with salt.utils.fopen(self.opts['reactor']) as fp_:
                     react_map = yaml.safe_load(fp_.read())
@@ -549,7 +640,7 @@ class Reactor(multiprocessing.Process, salt.state.Compiler):
                 continue
             if len(ropt) != 1:
                 continue
-            key = ropt.keys()[0]
+            key = ropt.iterkeys().next()
             val = ropt[key]
             if fnmatch.fnmatch(tag, key):
                 if isinstance(val, string_types):
@@ -570,7 +661,9 @@ class Reactor(multiprocessing.Process, salt.state.Compiler):
         if high:
             errors = self.verify_high(high)
             if errors:
-                return errors
+                log.error(('Unable to render reactions for event {0} due to '
+                           'errors ({1}) in one or more of the sls files ({2})').format(tag, errors, reactors))
+                return []  # We'll return nothing since there was an error
             chunks = self.order_chunks(self.compile_high_data(high))
         return chunks
 
@@ -585,7 +678,12 @@ class Reactor(multiprocessing.Process, salt.state.Compiler):
         '''
         Enter into the server loop
         '''
+        salt.utils.appendproctitle(self.__class__.__name__)
+
+        # instantiate some classes inside our new process
         self.event = SaltEvent('master', self.opts['sock_dir'])
+        self.wrap = ReactWrap(self.opts)
+
         for data in self.event.iter_events(full=True):
             reactors = self.list_reactors(data['tag'])
             if not reactors:
@@ -599,8 +697,18 @@ class ReactWrap(object):
     '''
     Create a wrapper that executes low data for the reaction system
     '''
+    # class-wide cache of clients
+    client_cache = None
+
     def __init__(self, opts):
         self.opts = opts
+        if ReactWrap.client_cache is None:
+            ReactWrap.client_cache = salt.utils.cache.CacheDict(opts['reactor_refresh_interval'])
+
+        self.pool = salt.utils.process.ThreadPool(
+            self.opts['reactor_worker_threads'],  # number of workers for runner/wheel
+            queue_size=self.opts['reactor_worker_hwm']  # queue size for those workers
+        )
 
     def run(self, low):
         '''
@@ -608,37 +716,40 @@ class ReactWrap(object):
         LowData
         '''
         l_fun = getattr(self, low['state'])
-        f_call = salt.utils.format_call(l_fun, low)
         try:
-            ret = l_fun(*f_call.get('args', ()), **f_call.get('kwargs', {}))
+            f_call = salt.utils.format_call(l_fun, low)
+            l_fun(*f_call.get('args', ()), **f_call.get('kwargs', {}))
         except Exception:
             log.error(
                     'Failed to execute {0}: {1}\n'.format(low['state'], l_fun),
                     exc_info=True
                     )
-            return False
-        return ret
 
-    def cmd(self, *args, **kwargs):
+    def local(self, *args, **kwargs):
         '''
         Wrap LocalClient for running :ref:`execution modules <all-salt.modules>`
         '''
-        local = salt.client.LocalClient(self.opts['conf_file'])
-        return local.cmd_async(*args, **kwargs)
+        if 'local' not in self.client_cache:
+            self.client_cache['local'] = salt.client.LocalClient(self.opts['conf_file'])
+        self.client_cache['local'].cmd_async(*args, **kwargs)
+
+    cmd = local
 
     def runner(self, fun, **kwargs):
         '''
         Wrap RunnerClient for executing :ref:`runner modules <all-salt.runners>`
         '''
-        runner = salt.runner.RunnerClient(self.opts)
-        return runner.low(fun, kwargs)
+        if 'runner' not in self.client_cache:
+            self.client_cache['runner'] = salt.runner.RunnerClient(self.opts)
+        self.pool.fire_async(self.client_cache['runner'].low, args=(fun, kwargs))
 
     def wheel(self, fun, **kwargs):
         '''
         Wrap Wheel to enable executing :ref:`wheel modules <all-salt.wheel>`
         '''
-        wheel = salt.wheel.Wheel(self.opts)
-        return wheel.call_func(fun, **kwargs)
+        if 'wheel' not in self.client_cache:
+            self.client_cache['wheel'] = salt.wheel.Wheel(self.opts)
+        self.pool.fire_async(self.client_cache['wheel'].low, args=(fun, kwargs))
 
 
 class StateFire(object):
@@ -659,7 +770,9 @@ class StateFire(object):
         '''
         Fire an event off on the master server
 
-        CLI Example::
+        CLI Example:
+
+        .. code-block:: bash
 
             salt '*' event.fire_master 'stuff to be in the event' 'tag'
         '''
